# Author: echel0n <echel0n@sickrage.ca>
# URL: https://sickrage.ca
# Git: https://git.sickrage.ca/SiCKRAGE/sickrage.git
#
# This file is part of SiCKRAGE.
#
# SiCKRAGE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# SiCKRAGE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiCKRAGE.  If not, see <http://www.gnu.org/licenses/>.
import threading
import time
from datetime import datetime, timedelta

from sqlalchemy import orm

import sickrage
from sickrage.core.databases.cache import CacheDB
from sickrage.core.helpers import full_sanitize_scene_name


class NameCache(object):
    def __init__(self, *args, **kwargs):
        self.name = "NAMECACHE"
        self.running = False
        self.min_time = 10
        self.last_update = {}
        self.cache = {}

    def should_update(self, show):
        # if we've updated recently then skip the update
        if datetime.today() - (self.last_update.get(show.name) or datetime.fromtimestamp(
                int(time.mktime(datetime.today().timetuple())))) < timedelta(minutes=self.min_time):
            return True

    def put(self, name, series_id=0):
        """
        Adds the show & tvdb id to the scene_names table in cache db

        :param name: The show name to cache
        :param series_id: the TVDB id that this show should be cached with (can be None/0 for unknown)
        """

        session = sickrage.app.cache_db.session()

        # standardize the name we're using to account for small differences in providers
        name = full_sanitize_scene_name(name)

        self.cache[name] = int(series_id)

        try:
            session.query(CacheDB.SceneName).filter_by(name=name, series_id=int(series_id)).one()
        except orm.exc.NoResultFound:
            session.add(CacheDB.SceneName(**{
                'series_id': series_id,
                'name': name
            }))
        finally:
            session.commit()

    def get(self, name):
        """
        Looks up the given name in the scene_names table in cache db

        :param name: The show name to look up.
        :return: the TVDB id that resulted from the cache lookup or None if the show wasn't found in the cache
        """
        name = full_sanitize_scene_name(name)
        if name in self.cache:
            return int(self.cache[name])

    def clear(self, series_id=None, name=None):
        """
        Deletes all entries from the cache matching the series_id or name.
        """

        session = sickrage.app.cache_db.session()

        if any([series_id, name]):
            if series_id:
                session.query(CacheDB.SceneName).filter_by(series_id=series_id).delete()
                session.commit()
            elif name:
                session.query(CacheDB.SceneName).filter_by(name=name).delete()
                session.commit()

            for key, value in self.cache.copy().items():
                if value == series_id or key == name:
                    del self.cache[key]

    def load(self):
        session = sickrage.app.cache_db.session()
        self.cache = dict([(x.name, x.series_id) for x in session.query(CacheDB.SceneName)])

    def save(self):
        """
        Commit cache to database file
        """

        session = sickrage.app.cache_db.session()

        sql_t = []

        for name, series_id in self.cache.items():
            try:
                session.query(CacheDB.SceneName).filter_by(name=name, series_id=series_id).one()
            except orm.exc.NoResultFound:
                sql_t.append({
                    'series_id': series_id,
                    'name': name
                })

        session.bulk_insert_mappings(CacheDB.SceneName, sql_t)
        session.commit()



# -*- coding: latin-1 -*-
# Author: adaur <adaur.underground@gmail.com>
# URL: https://sickrage.ca
#
# This file is part of SiCKRAGE.
#
# SiCKRAGE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# SiCKRAGE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiCKRAGE.  If not, see <http://www.gnu.org/licenses/>.


import sickrage
from sickrage.core.caches.tv_cache import TVCache
from sickrage.core.helpers import try_int
from sickrage.search_providers import TorrentProvider


class XthorProvider(TorrentProvider):
    def __init__(self):
        super(XthorProvider, self).__init__("Xthor", "https://xthor.tk", True)

        self._urls.update({
            'search': "https://api.xthor.tk"
        })

        # custom settings
        self.custom_settings = {
            'passkey': '',
            'freeleech': False,
            'confirmed': False,
            'minseed': 0,
            'minleech': 0
        }

        self.subcategories = [433, 637, 455, 639]

        self.cache = TVCache(self)

    def _check_auth(self):
        if self.custom_settings['passkey']:
            return True

        sickrage.app.log.warning(f'Your authentication credentials for {self.name} are missing, check your config.')

        return False

    def search(self, search_strings, age=0, series_id=None, series_provider_id=None, season=None, episode=None, **kwargs):
        results = []

        # check for auth
        if not self._check_auth:
            return results

        for mode in search_strings:
            search_params = {
                'passkey': self.custom_settings['passkey']
            }

            if self.custom_settings['freeleech']:
                search_params['freeleech'] = 1

            sickrage.app.log.debug("Search Mode: %s" % mode)
            for search_string in search_strings[mode]:
                if mode != 'RSS':
                    sickrage.app.log.debug("Search string: %s " % search_string)
                    search_params['search'] = search_string

                resp = self.session.get(self.urls['search'], params=search_params)
                if not resp or not resp.content:
                    sickrage.app.log.debug("No data returned from provider")
                    continue

                try:
                    data = resp.json()
                except ValueError:
                    sickrage.app.log.debug("No data returned from provider")
                    continue

                results += self.parse(data, mode)

        return results

    def parse(self, data, mode, **kwargs):
        """
        Parse search results from data
        :param data: response data
        :param mode: search mode
        :return: search results
        """

        results = []

        error_code = data.pop('error', {})
        if error_code.get('code'):
            if error_code.get('code') != 2:
                sickrage.app.log.warning('{0}', error_code.get('descr', 'Error code 2 - no description available'))
            return results

        account_ok = data.pop('user', {}).get('can_leech')
        if not account_ok:
            sickrage.app.log.warning('Sorry, your account is not allowed to download, check your ratio')
            return results

        torrent_rows = data.pop('torrents', {})

        if not torrent_rows:
            sickrage.app.log.debug('Provider has no results for this search')
            return results

        for row in torrent_rows:
            try:
                title = row.get('name')
                download_url = row.get('download_link')
                if not all([title, download_url]):
                    continue

                seeders = try_int(row.get('seeders'))
                leechers = try_int(row.get('leechers'))

                size = try_int(row.get('size'), -1)

                results += [
                    {'title': title, 'link': download_url, 'size': size, 'seeders': seeders, 'leechers': leechers}
                ]

                if mode != 'RSS':
                    sickrage.app.log.debug("Found result: {}".format(title))
            except Exception:
                sickrage.app.log.error("Failed parsing provider.")

        return results



"""Initial migration

Revision ID: 5
Revises:
Create Date: 2017-12-29 14:39:27.854291

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '5'
down_revision = '4'


def upgrade():
    pass


def downgrade():
    pass



# ##############################################################################
#  Author: echel0n <echel0n@sickrage.ca>
#  URL: https://sickrage.ca/
#  Git: https://git.sickrage.ca/SiCKRAGE/sickrage.git
#  -
#  This file is part of SiCKRAGE.
#  -
#  SiCKRAGE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  -
#  SiCKRAGE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  -
#  You should have received a copy of the GNU General Public License
#  along with SiCKRAGE.  If not, see <http://www.gnu.org/licenses/>.
# ##############################################################################


import importlib
import inspect
import os
import pkgutil
import re

import sickrage
from sickrage.core.helpers import is_ip_private


class NotificationProvider(object):
    def __init__(self):
        self.name = ""

        ### Notification Types
        self.NOTIFY_SNATCH = 1
        self.NOTIFY_DOWNLOAD = 2
        self.NOTIFY_SUBTITLE_DOWNLOAD = 3
        self.NOTIFY_GIT_UPDATE = 4
        self.NOTIFY_GIT_UPDATE_TEXT = 5
        self.NOTIFY_LOGIN = 6
        self.NOTIFY_LOGIN_TEXT = 7

        self.notifyStrings = {
            self.NOTIFY_SNATCH: _("Started Download"),
            self.NOTIFY_DOWNLOAD: _("Download Finished"),
            self.NOTIFY_SUBTITLE_DOWNLOAD: _("Subtitle Download Finished"),
            self.NOTIFY_GIT_UPDATE: _("SiCKRAGE Updated"),
            self.NOTIFY_GIT_UPDATE_TEXT: _("SiCKRAGE Updated To Commit#:"),
            self.NOTIFY_LOGIN: _("SiCKRAGE new login"),
            self.NOTIFY_LOGIN_TEXT: _("New login from IP: {0}. http://geomaplookup.net/?ip={0}")
        }

    @property
    def id(self):
        return str(re.sub(r"[^\w\d_]", "_", str(re.sub(r"[+]", "plus", self.name))).lower())

    @staticmethod
    def mass_notify_download(ep_name):
        for n in sickrage.app.notification_providers.values():
            try:
                n.notify_download(ep_name)
            except Exception:
                continue

    @staticmethod
    def mass_notify_subtitle_download(ep_name, lang):
        for n in sickrage.app.notification_providers.values():
            try:
                n.notify_subtitle_download(ep_name, lang)
            except Exception:
                continue

    @staticmethod
    def mass_notify_snatch(ep_name):
        for n in sickrage.app.notification_providers.values():
            try:
                n.notify_snatch(ep_name)
            except Exception:
                continue

    @staticmethod
    def mass_notify_version_update(new_version=""):
        if sickrage.app.config.general.notify_on_update:
            for n in sickrage.app.notification_providers.values():
                try:
                    n.notify_version_update(new_version)
                except Exception:
                    continue

    @staticmethod
    def mass_notify_login(ipaddress):
        if sickrage.app.config.general.notify_on_login and not is_ip_private(ipaddress):
            for n in sickrage.app.notification_providers.values():
                try:
                    n.notify_login(ipaddress)
                except Exception:
                    continue


class NotificationProviders(dict):
    def __init__(self):
        super(NotificationProviders, self).__init__()
        for (__, name, __) in pkgutil.iter_modules([os.path.dirname(__file__)]):
            imported_module = importlib.import_module('.' + name, package='sickrage.notification_providers')
            for __, klass in inspect.getmembers(imported_module,
                                                predicate=lambda o: inspect.isclass(o) and issubclass(o, NotificationProvider) and o is not NotificationProvider):
                self[klass().id] = klass()
                break



from __future__ import absolute_import, division, print_function

import logging

from six import add_metaclass
from trakt.client import TraktClient
from trakt.core.errors import ERRORS
from trakt.core.exceptions import RequestError, ClientError, ServerError
from trakt.helpers import has_attribute
from trakt.version import __version__  # NOQA

__all__ = (
    'Trakt',
    'RequestError',
    'ClientError',
    'ServerError',
    'ERRORS'
)


class TraktMeta(type):
    def __getattr__(self, name):
        if has_attribute(self, name):
            return super(TraktMeta, self).__getattribute__(name)

        if self.client is None:
            self.construct()

        return getattr(self.client, name)

    def __setattr__(self, name, value):
        if has_attribute(self, name):
            return super(TraktMeta, self).__setattr__(name, value)

        if self.client is None:
            self.construct()

        setattr(self.client, name, value)

    def __getitem__(self, key):
        if self.client is None:
            self.construct()

        return self.client[key]


@add_metaclass(TraktMeta)
class Trakt(object):
    client = None

    @classmethod
    def construct(cls):
        cls.client = TraktClient()


# Set default logging handler to avoid "No handler found" warnings.
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())
